@@ -32,8 +32,11 @@ from .object_storage_dao import (
     BucketAlreadyExists,
     BucketError,
     BucketNotFoundError,
+    MultiPartUploadAbortError,
+    MultiPartUploadAlreadyExistsError,
     MultiPartUploadConfirmError,
     MultiPartUploadNotFoundError,
+    MultipleActiveUploadsError,
     ObjectAlreadyExistsError,
     ObjectError,
     ObjectNotFoundError,
@@ -127,6 +130,7 @@ def _format_s3_error_code(error_code: str):
 
 def _translate_s3_client_errors(
     source_exception: botocore.exceptions.ClientError,
+    upload_id: Optional[str] = None,
     bucket_id: Optional[str] = None,
     object_id: Optional[str] = None,
 ) -> Exception:
@@ -144,11 +148,17 @@ def _translate_s3_client_errors(
     elif error_code == "BucketAlreadyExists":
         exception = BucketAlreadyExists(bucket_id=bucket_id)
     elif error_code == "NoSuchKey":
-        exception = ObjectNotFoundError(object_id=object_id)
+        exception = ObjectNotFoundError(bucket_id=bucket_id, object_id=object_id)
     elif error_code == "ObjectAlreadyInActiveTierError":
-        exception = ObjectAlreadyExistsError(object_id=object_id)
+        exception = ObjectAlreadyExistsError(bucket_id=bucket_id, object_id=object_id)
     elif error_code == "ObjectAlreadyInActiveTierError":
-        exception = ObjectAlreadyExistsError(object_id=object_id)
+        exception = ObjectAlreadyExistsError(bucket_id=bucket_id, object_id=object_id)
+    elif error_code == "NoSuchUpload":
+        if upload_id is None or bucket_id is None or object_id is None:
+            raise ValueError()
+        exception = MultiPartUploadNotFoundError(
+            upload_id=upload_id, bucket_id=bucket_id, object_id=object_id
+        )
     else:
         # exact match not found, match by keyword:
         if "Bucket" in error_code:
@@ -211,7 +221,7 @@ class ObjectStorageS3(ObjectStorageDao):  # pylint: disable=too-many-instance-at
     def __repr__(self) -> str:
         return f"ObjectStorageS3(config=S3ConfigBase(s3_endpoint_url={self.endpoint_url}, ...))"
 
-    def __enter__(self) -> ObjectStorageDao:
+    def __enter__(self) -> "ObjectStorageS3":
         """Setup storage connection/session."""
 
         self._client = boto3.client(
@@ -407,17 +417,20 @@ class ObjectStorageS3(ObjectStorageDao):  # pylint: disable=too-many-instance-at
             url=presigned_url["url"], fields=presigned_url["fields"]
         )
 
-    def _assert_multipart_upload_exist(
-        self, upload_id: str, bucket_id: str, object_id: str
-    ) -> None:
-        """Checks if a multipart upload with the given ID exists and whether it maps
-        to the specified object and bucket. Otherwise, raises UploadNotExistError.
+    def _list_mulitpart_upload_for_object(
+        self, bucket_id: str, object_id: str
+    ) -> list[str]:
+        """Lists all active multi-part upload for the given object. Returns a list of
+        their IDs.
+
+        (S3 allows multiple ongoing multi-part uploads.)
         """
+
         if not isinstance(self._client, botocore.client.BaseClient):
             raise OutOfContextError()
 
         try:
-            upload_list = self._client.list_multipart_uploads(
+            uploads_info = self._client.list_multipart_uploads(
                 Bucket=bucket_id,
             )
         except botocore.exceptions.ClientError as error:
@@ -425,28 +438,67 @@ class ObjectStorageS3(ObjectStorageDao):  # pylint: disable=too-many-instance-at
                 error, bucket_id=bucket_id, object_id=object_id
             ) from error
 
-        for upload in upload_list["Uploads"]:
-            if upload["UploadId"] == upload_id:
-                if upload["Key"] == object_id:
-                    return
-                raise MultiPartUploadNotFoundError(
-                    upload_id=upload_id,
-                    bucket_id=bucket_id,
-                    object_id=object_id,
-                    details="Object ID missmatch",
-                )
+        if "Uploads" in uploads_info:
+            upload_list = uploads_info["Uploads"]
+            return [
+                upload["UploadId"]
+                for upload in upload_list
+                if upload["Key"] == object_id
+            ]
 
-        raise MultiPartUploadNotFoundError(
-            upload_id=upload_id,
-            bucket_id=bucket_id,
-            object_id=object_id,
+        return []
+
+    def _assert_no_multipart_upload(self, bucket_id: str, object_id: str):
+        """Ensure that there are no active multi-part uploads for the given object."""
+
+        upload_ids = self._list_mulitpart_upload_for_object(
+            bucket_id=bucket_id, object_id=object_id
         )
+        if len(upload_ids) > 0:
+            raise MultiPartUploadAlreadyExistsError(
+                bucket_id=bucket_id, object_id=object_id
+            )
+
+    def _assert_multipart_upload_exist(
+        self,
+        upload_id: str,
+        bucket_id: str,
+        object_id: str,
+        assert_exclusiveness: bool = True,
+    ) -> None:
+        """Checks if a multipart upload with the given ID exists and whether it maps
+        to the specified object and bucket. Otherwise, raises UploadNotExistError.
+
+        By default, it is also verified that this upload is the only upload active for
+        that file. Otherwise, raises
+        """
+        if not isinstance(self._client, botocore.client.BaseClient):
+            raise OutOfContextError()
+
+        upload_ids = self._list_mulitpart_upload_for_object(
+            bucket_id=bucket_id, object_id=object_id
+        )
+        n_uploads = len(upload_ids)
+
+        if assert_exclusiveness and n_uploads > 1:
+            raise MultipleActiveUploadsError(
+                bucket_id=bucket_id, object_id=object_id, upload_ids=upload_ids
+            )
+
+        if upload_id not in upload_ids:
+            raise MultiPartUploadNotFoundError(
+                upload_id=upload_id,
+                bucket_id=bucket_id,
+                object_id=object_id,
+            )
 
     def init_multipart_upload(self, bucket_id: str, object_id: str) -> str:
         """Initiates a mulipart upload procedure. Returns the upload ID."""
         if not isinstance(self._client, botocore.client.BaseClient):
             raise OutOfContextError()
 
+        self._assert_no_multipart_upload(bucket_id=bucket_id, object_id=object_id)
+
         try:
             response = self._client.create_multipart_upload(
                 Bucket=bucket_id, Key=object_id
@@ -492,7 +544,7 @@ class ObjectStorageS3(ObjectStorageDao):  # pylint: disable=too-many-instance-at
             )
         except botocore.exceptions.ClientError as error:
             raise _translate_s3_client_errors(
-                error, bucket_id=bucket_id, object_id=object_id
+                error, upload_id=upload_id, bucket_id=bucket_id, object_id=object_id
             ) from error
 
     def _get_parts_info(
@@ -511,16 +563,15 @@ class ObjectStorageS3(ObjectStorageDao):  # pylint: disable=too-many-instance-at
         )
 
         try:
-            parts_info = self._client.list_parts(
+            return self._client.list_parts(
                 Bucket=bucket_id,
                 Key=object_id,
                 UploadId=upload_id,
             )
         except botocore.exceptions.ClientError as error:
             raise _translate_s3_client_errors(
-                error, bucket_id=bucket_id, object_id=object_id
+                error, upload_id=upload_id, bucket_id=bucket_id, object_id=object_id
             ) from error
-        return parts_info
 
     # pylint: disable=too-many-arguments
     def _check_uploaded_parts(
@@ -604,6 +655,61 @@ class ObjectStorageS3(ObjectStorageDao):  # pylint: disable=too-many-instance-at
                     + f" first part {first_part_size}.",
                 )
 
+    # pylint: disable=too-many-arguments
+    def abort_multipart_upload(
+        self,
+        upload_id: str,
+        bucket_id: str,
+        object_id: str,
+    ) -> None:
+        """Abort a multipart upload with the specified ID. All uploaded content is
+        deleted.
+        """
+
+        if not isinstance(self._client, botocore.client.BaseClient):
+            raise OutOfContextError()
+
+        self._assert_multipart_upload_exist(
+            upload_id=upload_id,
+            bucket_id=bucket_id,
+            object_id=object_id,
+            assert_exclusiveness=False,
+        )
+        # Exclusiveness is not enforced here since the abortion of an upload might be
+        # used to resolve the invalid state of multiple uploads for the same object.
+
+        try:
+            self._client.abort_multipart_upload(
+                Bucket=bucket_id,
+                Key=object_id,
+                UploadId=upload_id,
+            )
+        except botocore.exceptions.ClientError as error:
+            raise _translate_s3_client_errors(
+                error, upload_id=upload_id, bucket_id=bucket_id, object_id=object_id
+            ) from error
+
+        # verify that the abortion was successful as recommended by the boto3
+        # documentation:
+        # (Please see here for more details:
+        # https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3.html?highlight=abort_multipart_upload#S3.Client.abort_multipart_upload
+        # )
+        try:
+            parts_info = self._get_parts_info(
+                upload_id=upload_id,
+                bucket_id=bucket_id,
+                object_id=object_id,
+            )
+        except MultiPartUploadNotFoundError:
+            # this is proof enough that the upload was aborted:
+            return
+
+        # verify that no parts are remaining:
+        if "Parts" in parts_info and len(parts_info["Parts"]) > 0:
+            raise MultiPartUploadAbortError(
+                upload_id=upload_id, bucket_id=bucket_id, object_id=object_id
+            )
+
     # pylint: disable=too-many-arguments
     def complete_multipart_upload(
         self,
@@ -654,7 +760,7 @@ class ObjectStorageS3(ObjectStorageDao):  # pylint: disable=too-many-instance-at
             )
         except botocore.exceptions.ClientError as error:
             raise _translate_s3_client_errors(
-                error, bucket_id=bucket_id, object_id=object_id
+                error, upload_id=upload_id, bucket_id=bucket_id, object_id=object_id
             ) from error
 
     def get_object_download_url(
